########################################################################
# mdi.tcl
# A Microsoft Multiple Document Interface for Tcl/Tk 8.0 or later
# COPYRIGHT (C) 1998 by Mysund Scripts
#
#
# "One of these days i'll start commenting my code"
#
# Do NOT alter the following array even if you know what you are doing #
########################################################################
array set "Mysund_MDI" {
     "version"   "1.00.0"
     "name"      "::MDI"
     "platform"  "windows"
     "state"     "normal"
     "altered"   "no"
     "debug"     "no"
     "namespace" "yes"
     "proc"      "yes"
     }
#######################################################################
namespace eval $Mysund_MDI(name) {
namespace export master slave
set sysblue  #00007f
set sysgray  #7f7f7f
set syslgray #c0c0c0
set syswhite #ffffff
set sysbgray #707070
set imglist "file close min max norm mclose mmin mnorm"
foreach img $imglist {image create photo p$img -file $img.gif}
catch {font delete MDIfont}
font create MDIfont -family {MS Sans Serif} -weight bold -size 10

#######################################################################
# Make a master. Make a global array var called $name::
#######################################################################
proc master {name parent {args ""}} {
     set canvasargs [initmaster $name $parent $args]
     eval [subst "canvas $parent.$name $canvasargs"]
     pack $parent.$name -side top -fill both -expand yes
     menu $parent.$name.menu -tearoff 0
          $parent.$name.menu  add command -label "Minimize all"  -command [subst {::MDI::minall ::${name}::}]
          $parent.$name.menu  add command -label "Arrange icons" -command [subst {::MDI::arrall ::${name}::}]
          $parent.$name.menu  add command -label "Restore all"   -command [subst {::MDI::resall ::${name}::}]
          $parent.$name.menu  add command -label "Next"          -command [subst {::MDI::next   ::${name}::}]
          $parent.$name.menu  add command -label "Previous"      -command [subst {::MDI::prev   ::${name}::}]
          $parent.$name.menu  add separator
     bind $parent.$name <3> [subst {$parent.$name.menu post %X %Y}]
     #::MDI::checkresize ::${name}::
     wm protocol [winfo toplevel $parent] WM_DELETE_WINDOW [subst {::MDI::closemaster ::${name}::}]
     return $name
     }

#######################################################################
# Make a client. Make a global array var called $parent::$name
#######################################################################
proc slave {name parent args} {
     set mstructname ::${parent}::
     set cstructname ::${parent}::${name}
     upvar #0 $cstructname cstruct $mstructname mstruct
     set frameargs [initslave $name $mstructname $args]
     set cstruct(id) [eval [subst {$mstruct(path) create window $cstruct(rootx) $cstruct(rooty) $frameargs }]]
     MDI::setcurrent $name $mstructname
     eval [subst {bind $cstruct(path).system.title <ButtonPress-1>   {::MDI::startmove $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system.title <B1-Motion>       {::MDI::domove    $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system.title <ButtonRelease-1> {::MDI::endmove   $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system <ButtonPress-1>         {::MDI::startmove $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system <B1-Motion>             {::MDI::domove    $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system <ButtonRelease-1>       {::MDI::endmove   $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path).system.pfile <1>               {$cstruct(path).system.pfile.menu post \[winfo rootx $cstruct(userpath)\] \[winfo rooty $cstruct(userpath)\]}}]
     eval [subst {bind $cstruct(path) <Enter>  {::MDI::setclientcursor  $name $parent %x %y}}]
     eval [subst {bind $cstruct(path) <Motion> {::MDI::setclientcursor  $name $parent %x %y}}]
     eval [subst {bind $cstruct(path) <ButtonPress-1>   {::MDI::startresize $name $parent %x %y %X %Y}}]
     eval [subst {bind $cstruct(path) <B1-Motion>       {::MDI::doresize    $name $parent %X %Y}}]
     eval [subst {bind $cstruct(path) <ButtonRelease-1> {::MDI::endresize   $name $parent %X %Y}}]
     return $mstruct(path).$name.$name
     }

#######################################################################
# additional master initialization
#######################################################################
proc initmaster {name parent {args ""}} {
     if ![catch {namespace children ::$name}] {
          namespace delete ::$name
          }
     namespace eval ::$name {}
     upvar #0 ::${name}:: mstruct
     array set mstruct [subst {
           "path"        "$parent.$name"
           "clientlist"  ""
           "minwidth"    "140"
           "minheight"   "25"
           "menupath"	 ""
           "size"        "[lrange [split [winfo geometry [winfo toplevel $parent] ] +x] 0 1]"
           }]
     set args [string range $args 1 [expr [string length $args]-2]]
     foreach {option val} $args {
             switch -- $option {
                    -menupath    {set mstruct(menupath)    $val}
                    }
             }
     $parent con -bd 2 -relief sunken
     return "-bd 0 -relief sunken -bg $::MDI::sysbgray -highlightthickness 0 " ;# -height $mstruct(height) -width $mstruct(width)"
     }

#######################################################################
# additional slave initialization
#######################################################################
proc initslave {name parent args} {
     upvar #0 $parent$name cstruct $parent mstruct
     array set cstruct [subst {
           "this"      "$parent$name"
           "path"      "$mstruct(path).$name"
           "master"    "$parent"
           "rootx"     "[expr 0 + [llength $mstruct(clientlist)]*30 ]"
           "rooty"     "[expr 0 + [llength $mstruct(clientlist)]*30 ]"
           "height"    "300"
           "width"     "300"
           "title"     "No title"
           "userpath"  "$mstruct(path).$name.$name"
           "closecmd"  ""
           "minroot"   ""
           "status"    "normal"
           "lastevent" "-1 -1"
           "eventtype" ""}]
     set args [string range $args 1 [expr [string length $args]-2]]
     foreach {option val} $args {
             switch -- $option {
                    -x           {set cstruct(rootx)    $val}
                    -y           {set cstruct(rooty)    $val}
                    -height      {set cstruct(height)   $val}
                    -width       {set cstruct(width)    $val}
                    -closecmd    {set cstruct(closecmd) $val}
                    -tags        {set cstruct(tags)     $val}
                    -title       {set cstruct(title)    $val}
                    }
             }
     frame $cstruct(path) -bd 3 -relief raised
     pack  $cstruct(path) -side top -fill both -expand yes
     frame $cstruct(path).system -bg $::MDI::sysblue -bd 0 -cursor arrow
     pack  $cstruct(path).system -side top -fill x -padx 0 -pady 0
           set cb [button $cstruct(path).system.pclose -image pclose -bd 1 -command "::MDI::closeclient $name $parent"]
           set Mb [button $cstruct(path).system.pmax   -image pmax   -bd 1 -command "::MDI::maxclient   $name $parent"]
           set mb [button $cstruct(path).system.pmin   -image pmin   -bd 1 -command "::MDI::minclient   $name $parent"]
           set fb [label  $cstruct(path).system.pfile  -image pfile  -bd 1 -bg $::MDI::sysblue]
           pack $cb -side right -padx 2
           pack $cb $Mb $mb -side right
           pack $fb -side left -padx 2
           set tl [label $cstruct(path).system.title -text $cstruct(title) -anchor w -bd 0 -bg $::MDI::sysblue -fg $::MDI::syswhite -font MDIfont]
           pack $tl -side left
     frame $cstruct(path).$name -cursor arrow -bd 0
     pack  $cstruct(path).$name -side top -fill both -expand yes
     menu  $cstruct(path).system.pfile.menu -tearoff false
           $cstruct(path).system.pfile.menu add command -label "Restore"  -command "::MDI::normclient $name $parent" -state disabled
           $cstruct(path).system.pfile.menu add command -label "Move" -state disabled
           $cstruct(path).system.pfile.menu add command -label "Resize" -state disabled
           $cstruct(path).system.pfile.menu add command -label "Minimize" -command "::MDI::minclient $name $parent"
           $cstruct(path).system.pfile.menu add command -label "Maximize" -command "::MDI::maxclient $name $parent"
           $cstruct(path).system.pfile.menu add separator
           $cstruct(path).system.pfile.menu add command -label "Close" -command "$mstruct(path).$name.system.pclose invoke"
           $mstruct(path).menu add command -label $cstruct(title) -command [subst "::MDI::setcurrent $name $parent"]
     set mstruct(clientlist) [lappend mstruct(clientlist) $name]
     set result "-width $cstruct(width) -height $cstruct(height) -window $cstruct(path) -anchor nw"
     eval [subst {bind all <1> {+if {\[string first $cstruct(path) %W\] == "0"} {::MDI::setcurrent $name $parent}}}]
     return $result
     }

##############################################################
# -Geometrics (Resizeclient)
##############################################################
proc startresize {name parent x y X Y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
if {$cstruct(status)=="minimized"} return
set cstruct(lastevent) "$X $Y"
set geo   [split [winfo geometry $cstruct(path)] +x]
set down  [expr [lindex $geo 1] -20]
set right [expr [lindex $geo 0] -20]
if {$x<20} { ;#westy
   if {$y<20} {
      set result nw
      } else {
      if {$y>$down} {set result sw} else {set result w}
      }
   } else { ;#NOT westy
   if {$x>$right} { ;#easty
      if {$y<20} {
         set result ne
         } else {
         if {$y>$down} {set result se} else {set result e}
         }
      } else { ;#NOT westy and  NOT easty
      if {$y>$down} {set result s} else {set result n}
      }
   }
set cstruct(eventtype) $result
}
proc doresize {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
if {$cstruct(status)=="minimized"} return
set oldx  [lindex $cstruct(lastevent) 0]
set oldy  [lindex $cstruct(lastevent) 1]
if {[winfo rootx $mstruct(path)] > $x} {set x [winfo rootx $mstruct(path)]}
if {[winfo rooty $mstruct(path)] > $y} {set y [winfo rooty $mstruct(path)]}
if {([winfo rootx $mstruct(path)]+[winfo width  $mstruct(path)]) < $x} {set x [expr [winfo width  $mstruct(path)]+[winfo rootx $mstruct(path)]]}
if {([winfo rooty $mstruct(path)]+[winfo height $mstruct(path)]) < $y} {set y [expr [winfo height $mstruct(path)]+[winfo rooty $mstruct(path)]]}
set maxx  [winfo width  $cstruct(path)]
set maxy  [winfo height $cstruct(path)]
set dx    [expr $x-$oldx]
set dy    [expr $y-$oldy]
switch $cstruct(eventtype) {
       n  {$mstruct(path) itemconfig $cstruct(id) -height [expr $maxy-$dy]; $mstruct(path) move $cstruct(id)  0 $dy}
       ne {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx+$dx] -height [expr $maxy-$dy]; $mstruct(path) move $cstruct(id)  0 $dy}
       e  {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx+$dx]}
       se {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx+$dx] -height [expr $maxy+$dy]}
       s  {$mstruct(path) itemconfig $cstruct(id) -height [expr $maxy+$dy]}
       sw {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx-$dx] -height [expr $maxy+$dy]; $mstruct(path) move $cstruct(id)  $dx 0}
       w  {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx-$dx]; $mstruct(path) move $cstruct(id)                           $dx 0}
       nw {$mstruct(path) itemconfig $cstruct(id) -width  [expr $maxx-$dx] -height [expr $maxy-$dy]; $mstruct(path) move $cstruct(id)  $dx $dy}
       }
set cstruct(lastevent) "$x $y"
}
proc endresize {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
if {($cstruct(status)=="minimized") || ($cstruct(status)=="maximized")} return
update idletasks
array set cstruct [subst {
      "lastevent" "-1 -1"
      "eventtype" ""
      "rootx"     "[lindex [$mstruct(path) bbox $cstruct(id)] 0]"
      "rooty"     "[lindex [$mstruct(path) bbox $cstruct(id)] 1]"
      "width"     "[winfo width  $cstruct(path)]"
      "height"    "[winfo height $cstruct(path)]"
      }]
}
proc setclientcursor {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
set geo [split [winfo geometry $cstruct(path)] +x]
set right [expr [lindex $geo 0] - 20]
set down  [expr [lindex $geo 1] - 20]
set rightbd [expr [lindex $geo 0] - 7]
set downbd  [expr [lindex $geo 1] - 7]
if {("$cstruct(status)" == "minimized") || ("$cstruct(status)" == "maximized")} {
   $cstruct(path) configure -cursor arrow
   return
   }
if { (3 < $x) && ($x < $rightbd) && (3 < $y) && ($y < $downbd)} {
   $cstruct(path) configure -cursor "arrow"
   return
   }
if {"$x" < "20"} {
   if {"$y" < "20"} {$cstruct(path) configure -cursor size_nw_se ;return}
   if {"$y" > "$down"} {$cstruct(path) configure -cursor size_ne_sw ;return
      } else {$cstruct(path) configure -cursor size_we ;return}
   }
if {"$x" > "$right"} {
   if {"$y" < "20"} {$cstruct(path) configure -cursor size_ne_sw ;return}
   if {"$y" > "$down"} {$cstruct(path) configure -cursor size_nw_se ;return
      } else { $cstruct(path) configure -cursor size_we ;return}
   } else {$cstruct(path) configure -cursor size_ns ;return}
}
##############################################################
# -Geometrics (Moveclient)
##############################################################
proc startmove {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
set cstruct(lastevent) "$x $y"
}
proc domove {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
set oldx  [lindex $cstruct(lastevent) 0]
set oldy  [lindex $cstruct(lastevent) 1]
if {[winfo rootx $mstruct(path)] > $x} {set x [winfo rootx $mstruct(path)]}
if {[winfo rooty $mstruct(path)] > $y} {set y [winfo rooty $mstruct(path)]}
if {([winfo rootx $mstruct(path)]+[winfo width  $mstruct(path)]) < $x} {set x [expr [winfo width  $mstruct(path)]+[winfo rootx $mstruct(path)]]}
if {([winfo rooty $mstruct(path)]+[winfo height $mstruct(path)]) < $y} {set y [expr [winfo height $mstruct(path)]+[winfo rooty $mstruct(path)]]}
$mstruct(path) move $cstruct(id) [expr $x-$oldx] [expr $y-$oldy]
set cstruct(lastevent) "$x $y"
}
proc endmove {name parent x y} {
upvar #0 ::${parent}:: mstruct ::${parent}::${name} cstruct
if {$cstruct(status)=="minimized"} {
   set cstruct(minroot) [lrange [$mstruct(path) bbox $cstruct(id)] 0 1]
   } else {
   set cstruct(rootx) [lindex [$mstruct(path) bbox $cstruct(id)] 0]
   set cstruct(rooty) [lindex [$mstruct(path) bbox $cstruct(id)] 1]
   }
set cstruct(lastevent) "-1 -1"
}

##############################################################
# -Client handling maximize minimize normalize
##############################################################
proc closeclient {name parent} {
upvar #0 $parent$name cstruct $parent mstruct
#update idletasks
if {[set cmd [info commands $cstruct(closecmd)]] != ""} { if ![eval [subst $cmd]] return }
destroy $cstruct(path)
$mstruct(path) delete $cstruct(id)
set mstruct(clientlist) [lreplace $mstruct(clientlist) [lsearch $mstruct(clientlist) $name] [lsearch $mstruct(clientlist) $name]]
for {set i 0} {$i<=[$mstruct(path).menu index last]} {incr i} {
    if {([$mstruct(path).menu type $i]!="command")||("[$mstruct(path).menu entrycget $i -label]"!="$cstruct(title)")} {
       continue
       } else {
       $mstruct(path).menu delete $i
       }
    }
unset cstruct
}

proc minclient {name parent} {
upvar #0 $parent$name cstruct $parent mstruct
$mstruct(path) itemconfig $cstruct(id) -width $mstruct(minwidth) -height $mstruct(minheight)
set canx  [winfo width  $mstruct(path)]
set cany  [winfo height $mstruct(path)]
set minx  $mstruct(minwidth)
set miny  $mstruct(minheight)
$cstruct(path).system.pfile.menu entryconfig 0 -state normal
$cstruct(path).system.pfile.menu entryconfig 3 -state disabled
$cstruct(path).system.pfile.menu entryconfig 4 -state normal
if {$cstruct(minroot) != ""} {
   $mstruct(path) coords $cstruct(id) [lindex $cstruct(minroot) 0] [lindex $cstruct(minroot) 1]
   } else {
   set found 0
   for     {set iy 0} {$iy < "[expr $cany-$miny]" } {incr iy [expr 1+$mstruct(minheight)]} {
       for {set ix 0} {$ix < "[expr $canx-$minx]" } {incr ix [expr 1+$mstruct(minwidth)]} {
           set overlappers [$mstruct(path) find overlapping [expr $ix] [expr $cany-$iy] [expr $ix+$minx] [expr $cany-$miny-$iy]]
           set found 1
           if {"[llength $overlappers]" == "0"} {
              break     
              } else {
              foreach cli $mstruct(clientlist) {
                      upvar #0 $parent$cli curcli
                      if {[lsearch $overlappers $curcli(id)]!="-1" && $curcli(status)=="minimized" } {
                         set found 0
                         break
                         }
                      } ;#foreach
              if $found break
              } ;#ifelse
           if $found break
           } ;#for
       if $found break
       } ;#for
       $mstruct(path) coords     $cstruct(id) [expr $ix] [expr $cany-$iy-$miny]
    } ;#if
if {$cstruct(status) == "maximized"} {
   $cstruct(path).system.pmin configure -image pnorm -command [subst {::MDI::maxclient $name $parent}]
   } else {
   $cstruct(path).system.pmin configure -image pnorm -command [subst {::MDI::normclient $name $parent}]
   }
$cstruct(path).system.pmax configure -image pmax  -command [subst {::MDI::maxclient   $name $parent}]
$cstruct(path).system.pmin configure -image pnorm -command [subst {::MDI::normclient  $name $parent}]
set cstruct(status) "minimized"
eval [subst {bind $cstruct(path).system <Double-ButtonPress-1> { $cstruct(path).system.pmin invoke}}]
eval [subst {bind $cstruct(path).system.title <Double-ButtonPress-1> { $cstruct(path).system.pmin invoke}}]
foreach cli $mstruct(clientlist) {
        upvar #0 $parent$cli curclient
        if {$curclient(status)!="minimized"} {
           ::MDI::setcurrent $cli $parent
           break
           }
        }
update idletasks
if {[font measure MDIfont $cstruct(title)] > [winfo width $cstruct(path).system.title]} {
   set str [lindex [$cstruct(path).system.title con -text] 4]
   while {[font measure MDIfont $str] > [winfo width $cstruct(path).system.title]} {
         set str "[string range $str 0 [expr [string length $str]-3]]."
         }
   $cstruct(path).system.title con -text $str
   }
} ;#end proc

proc maxclient {name parent} {
upvar #0 $parent$name cstruct $parent mstruct
$cstruct(path).system.pmax con -command ""
set cstruct(status) "maximized"
$cstruct(path).system.pfile.menu entryconfig 4 -state disabled
$cstruct(path).system.pfile.menu entryconfig 0 -state normal
$cstruct(path).system.pfile.menu entryconfig 3 -state normal
if {$mstruct(menupath) == ""} {
   $mstruct(path) coords $cstruct(id) 0 0
   $mstruct(path) itemconfig $cstruct(id) -width [winfo width $mstruct(path)] -height [winfo height $mstruct(path)]
   $cstruct(path).system.pmin configure -image pmin  -command [subst {::MDI::minclient  $name $parent}]
   $cstruct(path).system.pmax configure -image pnorm -command [subst {::MDI::normclient $name $parent}]
   } else {
   if {[winfo class $mstruct(menupath)] != "Frame"} {puts "error: \"$mstruct(menupath)\"'class isn't a Frame" }
   foreach dims [winfo children $mstruct(menupath)] {
           eval [subst {set tmp($dims) "[pack info $dims]"}]
           pack forget $dims
           }
   label $mstruct(menupath).pfile -image pfile -bd 1 -bg $::MDI::syslgray
         pack $mstruct(menupath).pfile -side left -padx 2
   set cb [button $mstruct(menupath).pclose -image pmclose -bd 1 -command "::MDI::mclient $name $parent close"]
   set nb [button $mstruct(menupath).pnorm  -image pmnorm  -bd 1 -command "::MDI::mclient $name $parent norm"]
   set mb [button $mstruct(menupath).pmin   -image pmmin   -bd 1 -command "::MDI::mclient $name $parent min"]
   pack $cb -side right -padx 2
   pack $nb $mb -side right
   foreach aname [array names tmp] {
           eval [subst {pack $aname $tmp($aname)}]
           }
   catch {unset tmp}
   $mstruct(path) coords     $cstruct(id) -3 -25
   $mstruct(path) itemconfig $cstruct(id) -height [expr 32+[winfo height $mstruct(path)]] -width [expr 6+[winfo width $mstruct(path)]]
   $cstruct(path).system.pfile.menu entryconfig 0 -state normal -command [subst {::MDI::mclient $name $parent norm}]
   $cstruct(path).system.pfile.menu entryconfig 3 -state normal -command [subst {::MDI::mclient $name $parent min}]
   $cstruct(path).system.pfile.menu entryconfig 6 -state normal -command [subst {::MDI::mclient $name $parent close}]
   eval [subst {bind $mstruct(menupath).pfile <ButtonPress-1> {
        $cstruct(path).system.pfile.menu post \[winfo rootx $mstruct(menupath).pfile\] \[expr \[winfo rooty $mstruct(menupath).pfile\]+\[winfo height $mstruct(menupath).pfile\]\]}}]
   }
}

proc normclient {name parent} {
upvar #0 $parent$name cstruct $parent mstruct
$cstruct(path).system.pfile.menu entryconfig 4 -state normal
$cstruct(path).system.pfile.menu entryconfig 0 -state disabled
$cstruct(path).system.pfile.menu entryconfig 3 -state normal
$cstruct(path).system.title con -text $cstruct(title)
$mstruct(path) itemconfig $cstruct(id) -width $cstruct(width) -height $cstruct(height)
$mstruct(path) coords $cstruct(id) $cstruct(rootx) $cstruct(rooty)
#update idletasks
$cstruct(path).system.pmin configure -image pmin -command [subst {::MDI::minclient $name $parent}]
$cstruct(path).system.pmax configure -image pmax -command [subst {::MDI::maxclient $name $parent}]
set cstruct(status) "normal"
bind $cstruct(path).system <Double-1> "$cstruct(path).system.pmin invoke"
bind $cstruct(path).system.title <Double-1> "$cstruct(path).system.pmin invoke"
}

proc mclient {name parent command} {
upvar #0 $parent$name cstruct $parent mstruct
switch $command {
       close  {
              if {[set cmd [info commands $cstruct(closecmd)]] != ""} {if ![eval [subst $cmd]] return}
              destroy $cstruct(path)
              $mstruct(path) delete $cstruct(id)
              set mstruct(clientlist) [lreplace $mstruct(clientlist) [lsearch $mstruct(clientlist) $name] [lsearch $mstruct(clientlist) $name]]
              for {set i 0} {$i<=[$mstruct(path).menu index last]} {incr i} {
                  if {([$mstruct(path).menu type $i]!="command")||("[$mstruct(path).menu entrycget $i -label]"!="$cstruct(title)")} {
                     continue
                     } else {
                     $mstruct(path).menu delete $i
                     }
                  }
              unset cstruct
              }
       min    {
              ::MDI::minclient $name $parent
              }
       norm   {
              $mstruct(path) coords     $cstruct(id) $cstruct(rootx) $cstruct(rooty)
              $mstruct(path) itemconfig $cstruct(id) -width $cstruct(width) -height $cstruct(height)
              $cstruct(path).system.pmin con -image pmin -command [subst {::MDI::minclient $name $parent}]
              $cstruct(path).system.pfile.menu entryconfig 0 -state disabled
              $cstruct(path).system.pfile.menu entryconfig 3 -state normal
              $cstruct(path).system.pfile.menu entryconfig 4 -state normal
              set cstruct(status) "normal"
              }
       }
foreach dims [winfo children $mstruct(menupath)] {
        if {"$dims" == "$mstruct(menupath).pfile"}  {destroy $dims}
        if {"$dims" == "$mstruct(menupath).pclose"} {destroy $dims}
        if {"$dims" == "$mstruct(menupath).pmin"}   {destroy $dims}
        if {"$dims" == "$mstruct(menupath).pnorm"}  {destroy $dims}
        }
}

#######################################################################
# -Misc procs
#######################################################################
proc setcurrent {name parent} {
upvar #0 $parent$name cstruct $parent mstruct
set index [lsearch $mstruct(clientlist) $name ]
set mstruct(clientlist) "$name [lreplace $mstruct(clientlist) $index $index]"
foreach cli $mstruct(clientlist) {
        if {"$cli" == "$name"} {
           $cstruct(path).system       con -bg $MDI::sysblue
           $cstruct(path).system.title con -bg $MDI::sysblue -fg $MDI::syswhite
           $cstruct(path).system.pfile con -bg $MDI::sysblue
           } else {
           upvar #0 $parent$cli newcurrentstruct
           $newcurrentstruct(path).system       con -bg $MDI::sysgray
           $newcurrentstruct(path).system.title con -bg $MDI::sysgray -fg $MDI::syslgray
           $newcurrentstruct(path).system.pfile con -bg $MDI::sysgray
           }
        }
raise $cstruct(path)
}
proc minall {master} {
upvar #0 $master mstruct
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        $mstruct(path) coords $cstruct(id) -1800 0
        }
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        ::MDI::minclient $cli $master
        }
}
proc arrall {master} {
upvar #0 $master mstruct
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        if {$cstruct(status)!="minimized"} continue
        $mstruct(path) coords $cstruct(id) -150 0
        }
#update idletasks
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        if {$cstruct(status)!="minimized"} continue
        set cstruct(minroot) ""
        ::MDI::minclient $cli $master
        }
}
proc resall {master} {
upvar #0 $master mstruct
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        if {$cstruct(status)=="normal"} continue
        ::MDI::normclient $cli $master
        }
}
proc next {master} {
upvar #0 $master mstruct
::MDI::setcurrent [lindex $mstruct(clientlist) [expr [llength $mstruct(clientlist)]-1]] $master
}
proc prev {master} {
upvar #0 $master mstruct
::MDI::setcurrent [lindex $mstruct(clientlist) 1] $master
}

proc closemaster {master} {
upvar #0 $master mstruct
flush stdout
foreach cli $mstruct(clientlist) {
        upvar #0 $master$cli cstruct
        $cstruct(path).system.pclose invoke
        }
if {[llength $mstruct(clientlist)]} {
   return
   } else {
   destroy [winfo toplevel $mstruct(path)]
   unset mstruct
   namespace delete $master
   }
}

proc checkresize {m} {
#this is to nasty
upvar #0 $m ms
if {"$ms(size)"=="[lrange [split [winfo geometry [winfo toplevel $ms(path)] ] +x] 0 1]"} {
   after 500 [subst {::MDI::checkresize $m}] ;# Sorry cant find the event for this 8-(
   } else {
   arrall $m
   set ms(size) [lrange [split [winfo geometry [winfo toplevel $ms(path)] ] +x] 0 1]
   after 1000 [subst {::MDI::checkresize $m}] ;# Sorry cant find the event for this 8-(
   }
}
##################
} ;#end namespace